C  *********************************************************************
C  *********************************************************************
C  ***                                                               ***
C  ***                                                               ***
C  ***   HHH    HH   EEEEEEEEE   LL              AA        SSSSS     ***
C  ***   HHH    HH   EEEEEEEEE   LL             AAAA      SSSSSSS    ***
C  ***   HHH    HH   EE          LL            AA  AA    SS     SS   ***
C  ***   HHH    HH   EE          LL           AA    AA   SS     SS   ***
C  ***   HHH    HH   EE          LL          AAA    AA   SS          ***
C  ***   HHH    HH   EE          LL          AAA    AA   SS          ***
C  ***   HHHHHHHHH   EEEEEEE     LL          AAA    AA     SSS       ***
C  ***   HHHHHHHHH   EEEEEEE     LL          AAA    AA       SSS     ***
C  ***   HHH    HH   EE          LL          AAA    AA          SS   ***
C  ***   HHH    HH   EE          LL          AAAAAAAAA          SS   ***
C  ***   HHH    HH   EE          LL          AAAAAAAAA   SS     SS   ***
C  ***   HHH    HH   EE          LL          AAA    AA   SS     SS   ***
C  ***   HHH    HH   EE          LL          AAA    AA   SS     SS   ***
C  ***   HHH    HH   EEEEEEEEE   LLLLLLLLL   AAA    AA    SSSSSSS    ***
C  ***   HHH    HH   EEEEEEEEE   LLLLLLLLL   AAA    AA     SSSSS     ***
C  ***                                                               ***
C  ***               coded by H. Murayama & I. Watanabe              ***
C  ***                          ver.  2.4                            ***
C  ***                       12th  Apr.  1992                        ***
C  ***                                                               ***
C  *** For the formalism and notations, see the following reference: ***
C  ***           H. Murayama, I. Watanabe and K. Hagiwara            ***
C  ***           "HELAS: HELicity Amplitude Subroutines              ***
C  ***               for Feynman diagram evaluation"                 ***
C  ***               KEK Report 91-11, December 1991                 ***
C  ***                                                               ***
C  *********************************************************************
C  *********************************************************************
C
C  The subroutines are named as follows.
C
C  External Lines:
C   | f >                     : fermion (flow-IN)            ==>  IXXXXX
C   < f |                     : fermion (flow-OUT)           ==>  OXXXXX
C   epsilon^mu , epsilon*^mu  : vector boson (initial,final) ==>  VXXXXX
C   S                         : scalar boson (initial,final) ==>  SXXXXX
C
C  Vertices:
C   < f' V f >            : amplitude        of   FFV vertex ==>  IOVXXX
C   | f' V f >            : flow-in  fermion from FFV vertex ==>  FVIXXX
C   < f V f' |            : flow-out fermion from FFV vertex ==>  FVOXXX
C   J^mu(< f' | V | f >)  : vector   current from FFV vertex ==>  JIOXXX
C                         : W3       current from FFV vertex ==>  J3XXXX
C   < f' S f >            : amplitude        of   FFS vertex ==>  IOSXXX
C   | f' S f >            : flow-in  fermion from FFS vertex ==>  FSIXXX
C   < f S f' |            : flow-out fermion from FFS vertex ==>  FSOXXX
C   J(< f' | S | f >)     : scalar   current from FFS vertex ==>  HIOXXX
C   Gamma(V1,V2,V3)         : amplitude      of   VVV vertex ==>  VVVXXX
C   J^mu(V':V1,V2)          : vector current from VVV vertex ==>  JVVXXX
C   Gamma(V1,V2,S)          : amplitude      of   VVS vertex ==>  VVSXXX
C   J^mu(V':V,S)            : vector current from VVS vertex ==>  JVSXXX
C   J(S:V1,V2)              : scalar current from VVS vertex ==>  HVVXXX
C   Gamma(V,S1,S2)          : amplitude      of   VSS vertex ==>  VSSXXX
C   J^mu(V:S1,S2)           : vector current from VSS vertex ==>  JSSXXX
C   J(S':V,S)               : scalar current from VSS vertex ==>  HVSXXX
C   Gamma(S1,S2,S3)         : amplitude      of   SSS vertex ==>  SSSXXX
C   J(S':S1,S2)             : scalar current from SSS vertex ==>  HSSXXX
C   Gamma(WM,WP,WM,WP) : amplitude of   4-point W+/W- vertex ==>  WWWWXX
C   J^mu(W':W1,W2,W3)  : W current from 4-point W+/W- vertex ==>  JWWWXX
C   Gamma(WM,W3,WP,W3) : amplitude of   4-point W/W3  vertex ==>  W3W3XX
C   J^mu(W':W1,W2,W3)  : W current from 4-point W/W3  vertex ==>  JW3WXX
C   Gamma(V1,V2,S1,S2)   : amplitude        of   VVSS vertex ==>  VVSSXX
C   J^mu(V':V,S1,S2)     : vector current   from VVSS vertex ==>  JVSSXX
C   J(S':V1,V2,S)        : scalar current   from VVSS vertex ==>  HVVSXX
C   Gamma(S1,S2,S3,S4)   : amplitude        of   SSSS vertex ==>  SSSSXX
C   J(S':S1,S2,S3)       : scalar current   from SSSS vertex ==>  HSSSXX
C
C  Special Vertices:
C   | e' A e- >               : initial electron with photon ==>  EAIXXX
C   < e+ A e' |               : initial positron with photon ==>  EAOXXX
C   J^mu(< e+ | A | e->)      : t-channel photon from e-/e+  ==>  JEEXXX
C
C  Utilities for Momentum Manipulations:
C   P^mu(energy,mass,costh,phi)          : set up 4-momentum ==>  MOMNTX
C   P1^mu & P2^mu   : set up two 4-momenta in 1 2 rest frame ==>  MOM2CX
C   P_boosted                  : Lorentz boost of 4-momentum ==>  BOOSTX
C   P_rotated                       : rotation of 4-momentum ==>  ROTXXX
C
C  Standard Model Coupling Constants:
C   for VVV,VVVV vertices                                    ==>  COUP1X
C   for FFV vertices                                         ==>  COUP2X
C   for VVS,SSS,VVSS,SSSS vertices                           ==>  COUP3X
C   for FFS vertices                                         ==>  COUP4X
C
C **********************************************************************
C
      SUBROUTINE IXXXXX(P,FMASS,NHEL,NSF , FI)
C
C This subroutine computes a fermion wavefunction with the flowing-IN
C fermion number.
C
C INPUT:
C       real    P(0:3)         : four-momentum of fermion
C       real    FMASS          : mass          of fermion
C       integer NHEL = -1 or 1 : helicity      of fermion
C       integer NSF  = -1 or 1 : +1 for particle, -1 for anti-particle
C
C OUTPUT:
C       complex FI(6)          : fermion wavefunction               |FI>
C
      COMPLEX FI(6),CHI(2)
      REAL    P(0:3),SF(2),SFOMEG(2),OMEGA(2),FMASS,PP,PP3,SQP0P3,SQM
      INTEGER NHEL,NSF,IP,IM,NH
C
      FI(5) = CMPLX(P(0),P(3))*NSF
      FI(6) = CMPLX(P(1),P(2))*NSF
C
      NH=NHEL*NSF
C
      IF (FMASS.EQ.0.) GOTO 10
C
      PP=MIN(P(0),SQRT(P(1)**2+P(2)**2+P(3)**2))
C
      IF (PP.EQ.0.) GOTO 20
C
      SF(1)=REAL(1+NSF+(1-NSF)*NH)*.5
      SF(2)=REAL(1+NSF-(1-NSF)*NH)*.5
      OMEGA(1)=SQRT(P(0)+PP)
      OMEGA(2)=FMASS/OMEGA(1)
      IP=(3+NH)/2
      IM=(3-NH)/2
      SFOMEG(1)=SF(1)*OMEGA(IP)
      SFOMEG(2)=SF(2)*OMEGA(IM)
      PP3=MAX(PP+P(3),0.)
      CHI(1)=CMPLX( SQRT(PP3*.5/PP) )
      IF (PP3.EQ.0.) THEN
         CHI(2)=CMPLX( NH )
      ELSE
         CHI(2)=CMPLX( NH*P(1) , P(2) )/SQRT(2.*PP*PP3)
      ENDIF
C
      FI(1) = SFOMEG(1)*CHI(IM)
      FI(2) = SFOMEG(1)*CHI(IP)
      FI(3) = SFOMEG(2)*CHI(IM)
      FI(4) = SFOMEG(2)*CHI(IP)
C
      RETURN
C
  10  SQP0P3=SQRT(MAX(P(0)+P(3),0.))*NSF
      CHI(1)=CMPLX( SQP0P3 )
      IF (SQP0P3.EQ.0.) THEN
         CHI(2)=CMPLX( NHEL )*SQRT(2.*P(0))
      ELSE
         CHI(2)=CMPLX( NH*P(1), P(2) )/SQP0P3
      ENDIF
      IF (NH.EQ.1) THEN
         FI(1) = CMPLX( 0. )
         FI(2) = CMPLX( 0. )
         FI(3) = CHI(1)
         FI(4) = CHI(2)
      ELSE
         FI(1) = CHI(2)
         FI(2) = CHI(1)
         FI(3) = CMPLX( 0. )
         FI(4) = CMPLX( 0. )
      ENDIF
      RETURN
C
  20  SQM=SQRT(FMASS)
      IP=(1+NH)/2
      IM=(1-NH)/2
C
      FI(1) = IP     * SQM
      FI(2) = IM*NSF * SQM
      FI(3) = IP*NSF * SQM
      FI(4) = IM     * SQM
C
      RETURN
      END
